@@ -1,13 +1,12 @@
 package Nagios::MKLivestatus;
 
-use 5.008000;
+use 5.000000;
 use strict;
 use warnings;
-use IO::Socket;
 use Data::Dumper;
 use Carp;
 
-our $VERSION = '0.15';
+our $VERSION = '0.18';
 
 
 =head1 NAME
@@ -35,7 +34,8 @@ to install and activate the livestatus addon in your nagios installation.
 Creates an C<Nagios::MKLivestatus> object. C<new> takes at least the socketpath.
 Arguments are in key-value pairs.
 
-    socket                    path to the unix socket of check_mk livestatus
+    socket                    path to the UNIX socket of check_mk livestatus
+    server                    use this server for a TCP connection
     verbose                   verbose mode
     line_seperator            ascii code of the line seperator, defaults to 10, (newline)
     column_seperator          ascii code of the column seperator, defaults to 0 (null byte)
@@ -43,7 +43,7 @@ Arguments are in key-value pairs.
     host_service_seperator    ascii code of the host/service seperator, defaults to 124 (pipe)
 
 If the constructor is only passed a single argument, it is assumed to
-be a the C<socket> specification.
+be a the C<socket> specification. Use either socker OR server.
 
 =back
 
@@ -55,12 +55,14 @@ sub new {
     my(%options) = @_;
 
     my $self = {
-                    "verbose"                   => 0,
-                    "socket"                    => undef,
-                    "line_seperator"            => 10,   # defaults to newline
-                    "column_seperator"          => 0,    # defaults to null byte
-                    "list_seperator"            => 44,   # defaults to comma
-                    "host_service_seperator"    => 124,  # defaults to pipe
+                    "verbose"                   => 0,       # enable verbose output
+                    "socket"                    => undef,   # use unix sockets
+                    "server"                    => undef,   # use tcp connections
+                    "line_seperator"            => 10,      # defaults to newline
+                    "column_seperator"          => 0,       # defaults to null byte
+                    "list_seperator"            => 44,      # defaults to comma
+                    "host_service_seperator"    => 124,     # defaults to pipe
+                    "backend"                   => undef,   # should be keept undef, used internally
                };
     bless $self, $class;
 
@@ -73,8 +75,24 @@ sub new {
         }
     }
 
-    if(!defined $self->{'socket'}) {
-        croak('no socket given');
+    if(defined $self->{'socket'} and defined $self->{'server'}) {
+        croak('dont use socket and server at once');
+    }
+
+    if(!defined $self->{'socket'} and !defined $self->{'server'}) {
+        croak('please specify either socket or a server');
+    }
+
+    if(!defined $self->{'backend'}) {
+
+        if(defined $self->{'socket'}) {
+            use Nagios::MKLivestatus::UNIX;
+            $self->{'CONNECTOR'} = new Nagios::MKLivestatus::UNIX(%options);
+        }
+        elsif(defined $self->{'server'}) {
+            use Nagios::MKLivestatus::INET;
+            $self->{'CONNECTOR'} = new Nagios::MKLivestatus::INET(%options);
+        }
     }
 
     return $self;
@@ -138,6 +156,9 @@ sub selectall_arrayref {
     %{$opt} = map { lc $_ => $opt->{$_} } keys %{$opt};
 
     my $result = $self->_send($statement);
+    if(!defined $result) {
+        croak("got undef result for: $statement");
+    }
 
     # trim result set down to excepted row count
     if(defined $limit and $limit >= 1) {
@@ -186,7 +207,10 @@ sub selectall_hashref {
 
     my %indexed;
     for my $row (@{$result}) {
-        croak("key $key_field not found in result set") if !defined $row->{$key_field};
+        if(!defined $row->{$key_field}) {
+            my %possible_keys = keys %{$row};
+            croak("key $key_field not found in result set, possible keys are: ".join(', ', sort keys %possible_keys));
+        }
         $indexed{$row->{$key_field}} = $row;
     }
     return(\%indexed);
@@ -331,39 +355,29 @@ sub _send {
     my $statement = shift;
 
     croak("no statement") if !defined $statement;
-
-    if(!-S $self->{'socket'}) {
-        croak("failed to open socket $self->{'socket'}: $!");
-    }
-    my $sock = IO::Socket::UNIX->new($self->{'socket'});
-    if(!defined $sock or !$sock->connected()) {
-        croak("failed to connect to ($self->{'socket'}): $!");
-    }
-
-    my ($recv, @result);
     chomp($statement);
+
     my $send = "$statement\nSeparators: $self->{'line_seperator'} $self->{'column_seperator'} $self->{'list_seperator'} $self->{'host_service_seperator'}\n";
     print "> ".Dumper($send) if $self->{'verbose'};
-    print $sock $send;
-    $sock->shutdown(1) or croak("shutdown failed: $!");
-    while(<$sock>) { $recv .= $_; }
+    my $recv = $self->_send_socket($send);
     print "< ".Dumper($recv) if $self->{'verbose'};
-    close($sock);
-
     return if !defined $recv;
 
     my $line_seperator = chr($self->{'line_seperator'});
     my $col_seperator  = chr($self->{'column_seperator'});
 
-    for my $line (split/$line_seperator/, $recv) {
-        push @result, [ split/$col_seperator/, $line ];
+    my @result;
+    for my $line (split/$line_seperator/m, $recv) {
+        push @result, [ split/$col_seperator/m, $line ];
     }
 
     # for querys with column header, no seperate columns will be returned
     my $keys;
-    if($statement =~ m/^Columns: (.*)$/m) {
-        my @keys = split/\s+/, $1;
+    if($statement =~ m/^Columns:\ (.*)$/m) {
+        my @keys = split/\s+/m, $1;
         $keys = \@keys;
+    } elsif($statement =~ m/^Stats:\ (.*)$/m) {
+        @{$keys} = ($statement =~ m/^Stats: (.*)$/gm);
     } else {
         $keys = shift @result;
     }
@@ -371,6 +385,13 @@ sub _send {
     return({ keys => $keys, result => \@result});
 }
 
+########################################
+sub _send_socket {
+    my $self      = shift;
+    my $statement = shift;
+    my $recv = $self->{'CONNECTOR'}->_send_socket($statement);
+    return($recv);
+}
 
 1;
 
